#include <errno.h>

#include <stdio.h>
#include <stdlib.h>

#include <unistd.h>

#include <libgsloop.h>


static int client_cb(gsloop_t		*gsloop,
		     gsloop_client_t	*gsloop_client,
		     gsloop_event_t	*event,
		     void		*data)
{
	ssize_t		bytes_read;
	char		buf[255];
	int		err = 0;

	if (event->revents & POLLIN) {
		bytes_read = read(event->fd, buf, sizeof(buf));
		if (bytes_read == -1 || bytes_read == 0) {
			gsloop_client_close(gsloop, gsloop_client);
			return -errno;
		}
		buf[bytes_read] = 0x0;
		fprintf(stdout, "0x%08x: %s", (int)data, buf);
	} else {
		switch (event->revents) {
		case POLLOUT:
			printf("POLLOUT\n");
			break;
		default:
			printf("Event: %d\n", event->revents);
			break;
		}
	}

	return err;
}

int main()
{
	gsloop_t		gsloop;
	gsloop_attr_t		gsloop_attr = {
		.max_fd	= 32,
	};
	gsloop_server_attr_t	gsloop_server_attr = {
		.max_clients = 16,
	};
	int			err, loop = 5;

	err = gsloop_init(&gsloop, &gsloop_attr);

	err = gsloop_tcp_server_create(&gsloop, 4444, client_cb, (void *)0xdeadbeef, &gsloop_server_attr);

	while (err == 0 && loop--) {
		err = gsloop_dispatch(&gsloop, -1);
		printf("%d\n", loop);
	}

	gsloop_destroy(&gsloop);

	exit (EXIT_SUCCESS);
}
